﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Problems.VehicleRouting.Interfaces;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Core;
using HeuristicLab.Parameters;
using HeuristicLab.Data;
using HeuristicLab.Optimization;
using HeuristicLab.PluginInfrastructure;
using HeuristicLab.Problems.VehicleRouting.Variants;
using HeuristicLab.Common;

namespace HeuristicLab.Problems.VehicleRouting.ProblemInstances {
  [Item("CVRPProblemInstance", "Represents a single depot CVRP instance.")]
  [StorableClass]
  public class CVRPProblemInstance: SingleDepotVRPProblemInstance, IHomogenousCapacitatedProblemInstance {
    protected IValueParameter<DoubleValue> CapacityParameter {
      get { return (IValueParameter<DoubleValue>)Parameters["Capacity"]; }
    }
    protected IValueParameter<DoubleValue> OverloadPenaltyParameter {
      get { return (IValueParameter<DoubleValue>)Parameters["EvalOverloadPenalty"]; }
    }

    public DoubleValue Capacity {
      get { return CapacityParameter.Value; }
      set { CapacityParameter.Value = value; }
    }

    [Storable]
    private DoubleValue currentOverloadPenalty;

    public DoubleValue OverloadPenalty {
      get {
        if (currentOverloadPenalty != null)
          return currentOverloadPenalty;
        else
          return OverloadPenaltyParameter.Value; }
      set { currentOverloadPenalty = value; }
    }

    protected override IEnumerable<IOperator> GetOperators() {
        return base.GetOperators()
          .Where(o => o is IHomogenousCapacitatedOperator).Cast<IOperator>();
    }

    protected override IEnumerable<IOperator> GetAnalyzers() {
      return ApplicationManager.Manager.GetInstances<ICapacitatedOperator>()
        .Where(o => o is IAnalyzer)
        .Cast<IOperator>().Union(base.GetAnalyzers());
    }

    protected override IVRPEvaluator Evaluator {
      get {
        return new CVRPEvaluator();
      }
    }
    
    [StorableConstructor]
    protected CVRPProblemInstance(bool deserializing) : base(deserializing) { }

    public CVRPProblemInstance() {
      Parameters.Add(new ValueParameter<DoubleValue>("Capacity", "The capacity of each vehicle.", new DoubleValue(0)));
      Parameters.Add(new ValueParameter<DoubleValue>("EvalOverloadPenalty", "The overload penalty considered in the evaluation.", new DoubleValue(100)));

      AttachEventHandlers();
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new CVRPProblemInstance(this, cloner);
    }

    protected CVRPProblemInstance(CVRPProblemInstance original, Cloner cloner)
      : base(original, cloner) {
        currentOverloadPenalty = cloner.Clone(original.currentOverloadPenalty) as DoubleValue;

        AttachEventHandlers();
    }

    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserializationHook() {
      AttachEventHandlers();
    }

    private void AttachEventHandlers() {
      CapacityParameter.ValueChanged += new EventHandler(CapacityParameter_ValueChanged);
      CapacityParameter.Value.ValueChanged += new EventHandler(Capacity_ValueChanged);
      OverloadPenaltyParameter.ValueChanged += new EventHandler(OverloadPenaltyParameter_ValueChanged);
      OverloadPenaltyParameter.Value.ValueChanged += new EventHandler(OverloadPenalty_ValueChanged);
    }

    public override void InitializeState() {
      base.InitializeState();

      currentOverloadPenalty = null;
    }

    #region Event handlers
    void CapacityParameter_ValueChanged(object sender, EventArgs e) {
      CapacityParameter.Value.ValueChanged += new EventHandler(Capacity_ValueChanged);
      BestKnownSolution = null;
    }
    void Capacity_ValueChanged(object sender, EventArgs e) {
      BestKnownSolution = null;
    }
    void OverloadPenaltyParameter_ValueChanged(object sender, EventArgs e) {
      OverloadPenaltyParameter.Value.ValueChanged += new EventHandler(OverloadPenalty_ValueChanged);
      EvalBestKnownSolution();
    }
    void OverloadPenalty_ValueChanged(object sender, EventArgs e) {
      EvalBestKnownSolution();
    }
    #endregion
  }
}